@@ -41,10 +41,7 @@ impl FormData {
 }
 impl Default for FormData {
     fn default() -> Self {
-        FormData {
-            fields: MultiMap::new(),
-            files: MultiMap::new(),
-        }
+        Self::new()
     }
 }
 fn get_extension_from_filename(filename: &str) -> Option<&str> {
@@ -87,7 +84,7 @@ impl FilePart {
             TextNonce::sized_urlsafe(32).unwrap().into_string(),
             file_name
                 .as_deref()
-                .and_then(|f| get_extension_from_filename(&f))
+                .and_then(|f| get_extension_from_filename(f))
                 .unwrap_or("unknown")
         ));
         let mut file = File::create(&path).await?;
