@@ -6,7 +6,7 @@ use std::path::Path;
 use std::sync::Arc;
 
 use crate::http::Mime;
-use futures::{TryStream, TryStreamExt};
+use futures_util::{TryStream, TryStreamExt};
 use hyper::server::accept::{self, Accept};
 use hyper::server::conn::AddrIncoming;
 use hyper::Server as HyperServer;
@@ -17,7 +17,7 @@ use crate::tls::{TlsAcceptor, TlsConfigBuilder};
 use crate::transport::LiftIo;
 use crate::{Catcher, Router, Service};
 
-pub fn builder<I>(incoming: I) -> hyper::server::Builder<I> {
+fn builder<I>(incoming: I) -> hyper::server::Builder<I> {
     HyperServer::builder(incoming)
 }
 
@@ -267,14 +267,14 @@ impl TlsServer {
     ///
     /// *This function requires the `"tls"` feature.*
     pub fn key_path(self, path: impl AsRef<Path>) -> Self {
-        self.with_tls(|tls| tls.key_path(path))
+        self.with_tls(|tls| tls.with_key_path(path))
     }
 
     /// Specify the file path to read the certificate.
     ///
     /// *This function requires the `"tls"` feature.*
     pub fn cert_path(self, path: impl AsRef<Path>) -> Self {
-        self.with_tls(|tls| tls.cert_path(path))
+        self.with_tls(|tls| tls.with_cert_path(path))
     }
 
     /// Specify the file path to read the trust anchor for optional client authentication.
@@ -284,7 +284,7 @@ impl TlsServer {
     ///
     /// *This function requires the `"tls"` feature.*
     pub fn client_auth_optional_path(self, path: impl AsRef<Path>) -> Self {
-        self.with_tls(|tls| tls.client_auth_optional_path(path))
+        self.with_tls(|tls| tls.with_client_auth_optional_path(path))
     }
 
     /// Specify the file path to read the trust anchor for required client authentication.
@@ -294,21 +294,21 @@ impl TlsServer {
     ///
     /// *This function requires the `"tls"` feature.*
     pub fn client_auth_required_path(self, path: impl AsRef<Path>) -> Self {
-        self.with_tls(|tls| tls.client_auth_required_path(path))
+        self.with_tls(|tls| tls.with_client_auth_required_path(path))
     }
 
     /// Specify the in-memory contents of the private key.
     ///
     /// *This function requires the `"tls"` feature.*
     pub fn key(self, key: impl AsRef<[u8]>) -> Self {
-        self.with_tls(|tls| tls.key(key.as_ref()))
+        self.with_tls(|tls| tls.with_key(key.as_ref()))
     }
 
     /// Specify the in-memory contents of the certificate.
     ///
     /// *This function requires the `"tls"` feature.*
     pub fn cert(self, cert: impl AsRef<[u8]>) -> Self {
-        self.with_tls(|tls| tls.cert(cert.as_ref()))
+        self.with_tls(|tls| tls.with_cert(cert.as_ref()))
     }
 
     /// Specify the in-memory contents of the trust anchor for optional client authentication.
@@ -318,7 +318,7 @@ impl TlsServer {
     ///
     /// *This function requires the `"tls"` feature.*
     pub fn client_auth_optional(self, trust_anchor: impl AsRef<[u8]>) -> Self {
-        self.with_tls(|tls| tls.client_auth_optional(trust_anchor.as_ref()))
+        self.with_tls(|tls| tls.with_client_auth_optional(trust_anchor.as_ref()))
     }
 
     /// Specify the in-memory contents of the trust anchor for required client authentication.
@@ -328,14 +328,14 @@ impl TlsServer {
     ///
     /// *This function requires the `"tls"` feature.*
     pub fn client_auth_required(self, trust_anchor: impl AsRef<[u8]>) -> Self {
-        self.with_tls(|tls| tls.client_auth_required(trust_anchor.as_ref()))
+        self.with_tls(|tls| tls.with_client_auth_required(trust_anchor.as_ref()))
     }
 
     /// Specify the DER-encoded OCSP response.
     ///
     /// *This function requires the `"tls"` feature.*
     pub fn ocsp_resp(self, resp: impl AsRef<[u8]>) -> Self {
-        self.with_tls(|tls| tls.ocsp_resp(resp.as_ref()))
+        self.with_tls(|tls| tls.with_ocsp_resp(resp.as_ref()))
     }
 
     fn with_tls<Func>(self, func: Func) -> Self
@@ -420,21 +420,30 @@ impl TlsServer {
 mod tests {
     use crate::prelude::*;
 
-    #[tokio::main]
-    async fn test_hello_word() {
+    #[tokio::test]
+    async fn test_server() {
         #[fn_handler]
         async fn hello_world() -> Result<&'static str, ()> {
             Ok("Hello World")
         }
-        let router = Router::new().get(hello_world);
+        #[fn_handler]
+        async fn json(res: &mut Response) {
+            #[derive(Serialize, Debug)]
+            struct User {
+                name: String,
+            }
+            res.render_json(&User { name: "jobs".into() });
+        }
+        let router = Router::new().get(hello_world).push(Router::with_path("json").get(json));
 
         tokio::task::spawn(async {
-            Server::new(router).bind(([0, 0, 0, 0], 7878)).await;
+            Server::new(router).bind(([0, 0, 0, 0], 7979)).await;
         });
 
+        tokio::time::sleep(tokio::time::Duration::from_secs(5)).await;
         let client = reqwest::Client::new();
         let result = client
-            .get("http://0.0.0.0:7878")
+            .get("http://127.0.0.1:7979")
             .send()
             .await
             .unwrap()
@@ -442,5 +451,56 @@ mod tests {
             .await
             .unwrap();
         assert_eq!(result, "Hello World");
+
+        let client = reqwest::Client::new();
+        let result = client
+            .get("http://127.0.0.1:7979/json")
+            .send()
+            .await
+            .unwrap()
+            .text()
+            .await
+            .unwrap();
+        assert_eq!(result, r#"{"name":"jobs"}"#);
+
+        let result = client
+            .get("http://127.0.0.1:7979/not_exist")
+            .send()
+            .await
+            .unwrap()
+            .text()
+            .await
+            .unwrap();
+        assert!(result.contains("Not Found"));
+        let result = client
+            .get("http://127.0.0.1:7979/not_exist")
+            .header("accept", "application/json")
+            .send()
+            .await
+            .unwrap()
+            .text()
+            .await
+            .unwrap();
+        assert!(result.contains(r#""code":404"#));
+        let result = client
+            .get("http://127.0.0.1:7979/not_exist")
+            .header("accept", "text/plain")
+            .send()
+            .await
+            .unwrap()
+            .text()
+            .await
+            .unwrap();
+        assert!(result.contains("code:404"));
+        let result = client
+            .get("http://127.0.0.1:7979/not_exist")
+            .header("accept", "application/xml")
+            .send()
+            .await
+            .unwrap()
+            .text()
+            .await
+            .unwrap();
+        assert!(result.contains("<code>404</code>"));
     }
 }
